@@ -4,6 +4,8 @@
 use std::{collections::BTreeMap, iter};
 
 use serde::{Deserialize, Serialize};
+#[cfg(feature = "serde_json")]
+use serde_json::Value;
 
 use super::{
     build_fn, builder, from, new,
@@ -314,8 +316,23 @@ impl OperationBuilder {
     }
 
     /// Add or change parameters of the [`Operation`].
-    pub fn parameters<I: IntoIterator<Item = Parameter>>(mut self, parameters: Option<I>) -> Self {
-        set_value!(self parameters parameters.map(|parameters| parameters.into_iter().collect()))
+    pub fn parameters<I: IntoIterator<Item = P>, P: Into<Parameter>>(
+        mut self,
+        parameters: Option<I>,
+    ) -> Self {
+        self.parameters = parameters.map(|parameters| {
+            if let Some(mut params) = self.parameters {
+                params.extend(parameters.into_iter().map(|parameter| parameter.into()));
+                params
+            } else {
+                parameters
+                    .into_iter()
+                    .map(|parameter| parameter.into())
+                    .collect()
+            }
+        });
+
+        self
     }
 
     /// Append parameter to [`Operation`] parameters.
@@ -391,45 +408,78 @@ impl OperationBuilder {
     }
 }
 
-builder! {
-    ParameterBuilder;
-
-    /// Implements [OpenAPI Parameter Object][parameter] for [`Operation`].
+/// Implements [OpenAPI Parameter Object][parameter] for [`Operation`].
+///
+/// [parameter]: https://spec.openapis.org/oas/latest.html#parameter-object
+#[non_exhaustive]
+#[derive(Serialize, Deserialize, Default, Clone)]
+#[cfg_attr(feature = "debug", derive(Debug))]
+#[serde(rename_all = "camelCase")]
+pub struct Parameter {
+    /// Name of the parameter.
     ///
-    /// [parameter]: https://spec.openapis.org/oas/latest.html#parameter-object
-    #[non_exhaustive]
-    #[derive(Serialize, Deserialize, Default, Clone)]
-    #[cfg_attr(feature = "debug", derive(Debug))]
-    #[serde(rename_all = "camelCase")]
-    pub struct Parameter {
-        /// Name of the parameter.
-        ///
-        /// * For [`ParameterIn::Path`] this must in accordance to path templating.
-        /// * For [`ParameterIn::Query`] `Content-Type` or `Authorization` value will be ignored.
-        pub name: String,
+    /// * For [`ParameterIn::Path`] this must in accordance to path templating.
+    /// * For [`ParameterIn::Query`] `Content-Type` or `Authorization` value will be ignored.
+    pub name: String,
 
-        /// Parameter location.
-        #[serde(rename = "in")]
-        pub parameter_in: ParameterIn,
+    /// Parameter location.
+    #[serde(rename = "in")]
+    pub parameter_in: ParameterIn,
 
-        /// Markdown supported description of the parameter.
-        #[serde(skip_serializing_if = "Option::is_none")]
-        pub description: Option<String>,
+    /// Markdown supported description of the parameter.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub description: Option<String>,
 
-        /// Declares whether the parameter is required or not for api.
-        ///
-        /// * For [`ParameterIn::Path`] this must and will be [`Required::True`].
-        pub required: Required,
+    /// Declares whether the parameter is required or not for api.
+    ///
+    /// * For [`ParameterIn::Path`] this must and will be [`Required::True`].
+    pub required: Required,
 
-        /// Delcares the parameter deprecated status.
-        #[serde(skip_serializing_if = "Option::is_none")]
-        pub deprecated: Option<Deprecated>,
-        // pub allow_empty_value: bool, this is going to be removed from further open api spec releases
+    /// Delcares the parameter deprecated status.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub deprecated: Option<Deprecated>,
+    // pub allow_empty_value: bool, this is going to be removed from further open api spec releases
+    /// Schema of the parameter. Typically [`Component::Property`] is used.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub schema: Option<Component>,
 
-        /// Schema of the parameter. Typically [`Component::Property`] is used.
-        #[serde(skip_serializing_if = "Option::is_none")]
-        pub schema: Option<Component>,
-    }
+    /// Describes how [`Parameter`] is being serialized depending on [`Parameter::schema`] (type of a content).
+    /// Default value is based on [`ParameterIn`].
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub style: Option<ParameterStyle>,
+
+    /// When _`true`_ it will generate separate parameter value for each parameter with _`array`_ and _`object`_ type.
+    /// This is also _`true`_ by default for [`ParameterStyle::Form`].
+    ///
+    /// With explode _`false`_:
+    /// ```text
+    ///color=blue,black,brown
+    /// ```
+    ///
+    /// With explode _`true`_:
+    /// ```text
+    ///color=blue&color=black&color=brown
+    /// ```
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub explode: Option<bool>,
+
+    /// Defines wheter parameter should allow reserved characters defined by
+    /// [RFC3986](https://tools.ietf.org/html/rfc3986#section-2.2) _`:/?#[]@!$&'()*+,;=`_.
+    /// This is only applicable with [`ParameterIn::Query`]. Default value is _`false`_.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub allow_reserved: Option<bool>,
+
+    /// Example of [`Parameter`]'s potential value. This examples will override example
+    /// within [`Parameter::schema`] if defined.
+    #[cfg(feature = "serde_json")]
+    #[serde(skip_serializing_if = "Option::is_none")]
+    example: Option<Value>,
+
+    /// Example of [`Parameter`]'s potential value. This examples will override example
+    /// within [`Parameter::schema`] if defined.
+    #[cfg(not(feature = "serde_json"))]
+    #[serde(skip_serializing_if = "Option::is_none")]
+    example: Option<String>,
 }
 
 impl Parameter {
@@ -442,7 +492,39 @@ impl Parameter {
         }
     }
 }
+
+/// Builder for [`Parameter`] with chainable configuration methods to create a new [`Parameter`].
+#[derive(Default)]
+pub struct ParameterBuilder {
+    name: String,
+
+    parameter_in: ParameterIn,
+
+    description: Option<String>,
+
+    required: Required,
+
+    deprecated: Option<Deprecated>,
+
+    schema: Option<Component>,
+
+    style: Option<ParameterStyle>,
+
+    explode: Option<bool>,
+
+    allow_reserved: Option<bool>,
+
+    #[cfg(feature = "serde_json")]
+    example: Option<Value>,
+
+    #[cfg(not(feature = "serde_json"))]
+    example: Option<String>,
+}
+
+from!(Parameter ParameterBuilder name, parameter_in, description, required, deprecated, schema, style, explode, allow_reserved, example);
+
 impl ParameterBuilder {
+    new!(pub ParameterBuilder);
     /// Add name of the [`Parameter`].
     pub fn name<I: Into<String>>(mut self, name: I) -> Self {
         set_value!(self name name.into())
@@ -479,6 +561,35 @@ impl ParameterBuilder {
     pub fn schema<I: Into<Component>>(mut self, component: Option<I>) -> Self {
         set_value!(self schema component.map(|component| component.into()))
     }
+
+    /// Add or change serialization style of [`Parameter`].
+    pub fn style(mut self, style: Option<ParameterStyle>) -> Self {
+        set_value!(self style style)
+    }
+
+    /// Define whether [`Parameter`]s are exploded or not.
+    pub fn explode(mut self, explode: Option<bool>) -> Self {
+        set_value!(self explode explode)
+    }
+
+    /// Add or change whether [`Parameter`] should allow reserved characters.
+    pub fn allow_reserved(mut self, allow_reserved: Option<bool>) -> Self {
+        set_value!(self allow_reserved allow_reserved)
+    }
+
+    /// Add or change example of [`Parameter`]'s potential value.
+    #[cfg(not(feature = "serde_json"))]
+    pub fn example<I: Into<String>>(mut self, example: Option<I>) -> Self {
+        set_value!(self example example.map(|example| example.into()))
+    }
+
+    /// Add or change example of [`Parameter`]'s potential value.
+    #[cfg(feature = "serde_json")]
+    pub fn example(mut self, example: Option<Value>) -> Self {
+        set_value!(self example example)
+    }
+
+    build_fn!(pub Parameter name, parameter_in, required, description, deprecated, schema, style, explode, allow_reserved, example);
 }
 
 /// In definition of [`Parameter`].
@@ -501,3 +612,34 @@ impl Default for ParameterIn {
         Self::Path
     }
 }
+
+/// Defines how [`Parameter`] should be serialized.
+#[derive(Serialize, Deserialize, Clone)]
+#[cfg_attr(feature = "debug", derive(Debug))]
+#[serde(rename_all = "camelCase")]
+pub enum ParameterStyle {
+    /// Path style parameters defined by [RFC6570](https://tools.ietf.org/html/rfc6570#section-3.2.7)
+    /// e.g _`;color=blue`_.
+    /// Allowed with [`ParameterIn::Path`].
+    Matrix,
+    /// Lable style parameters defined by [RFC6570](https://datatracker.ietf.org/doc/html/rfc6570#section-3.2.5)
+    /// e.g _`.color=blue`_.
+    /// Allowed with [`ParameterIn::Path`].
+    Label,
+    /// Form style parameters defined by [RFC6570](https://datatracker.ietf.org/doc/html/rfc6570#section-3.2.8)
+    /// e.g. _`color=blue`_. Default value for [`ParameterIn::Query`] [`ParameterIn::Cookie`].
+    /// Allowed with [`ParameterIn::Query`] or [`ParameterIn::Cookie`].
+    Form,
+    /// Default value for [`ParameterIn::Path`] [`ParameterIn::Header`]. e.g. _`blue`_.
+    /// Allowed with [`ParameterIn::Path`] or [`ParameterIn::Header`].
+    Simple,
+    /// Space separated array values e.g. _`blue%20black%20brown`_.
+    /// Allowed with [`ParameterIn::Query`].
+    SpaceDelimited,
+    /// Pipe separated array values e.g. _`blue|black|brown`_.
+    /// Allowed with [`ParameterIn::Query`].
+    PipeDelimited,
+    /// Simple way of rendering nested objects using form parameters .e.g. _`color[B]=150`_.
+    /// Allowed with [`ParameterIn::Query`].
+    DeepObject,
+}
